@@ -16,6 +16,7 @@ import hashlib
 import re
 from os import path, getcwd, remove, makedirs, listdir
 from datetime import datetime, timedelta
+import tempfile
 from decorator import decorator
 import fasteners
 from dxf import DXFBase, DXF, hash_file, hash_bytes, split_digest
@@ -26,28 +27,41 @@ import securesystemslib.util
 import securesystemslib.settings
 securesystemslib.settings.HASH_ALGORITHMS = ['sha256']
 
-class _DTufConnection(object):
-    def __init__(self, url):
-        import tuf.settings
+class _TufResponse(object):
+    def __init__(self, url, request_kwargs):
         self._url = url
-        target = urlparse.urlparse(url).path.split('/')[-1]
-        pos = _skip_consistent_target_digest(target)
-        if pos == 0:
-            self._dgst = _updater_dxf.get_alias(target)[0]
-        else:
-            self._dgst = 'sha256:' + target[0:pos-1]
-        self._it, self._size = _updater_dxf.pull_blob(
-            self._dgst, size=True, chunk_size=tuf.settings.CHUNK_SIZE)
-        self._it = self._it.__iter__()
-        self._end = object()
+        self._request_kwargs = request_kwargs
+        self._it = None
         self._count = 0
-        if _updater_progress:
-            _updater_progress(self._dgst, b'', self._size)
 
-    def info(self):
-        return {'Content-Length': str(self._size)}
+    def __enter__(self):
+        return self
 
-    def read(self, _):
+    def raise_for_status(self):
+        #pylint: disable=attribute-defined-outside-init,protected-access
+        import tuf.settings
+        target = urlparse.urlparse(self._url).path.split('/')[-1]
+        pos = _skip_consistent_target_digest(target)
+        _updater_dxf._request_kwargs = self._request_kwargs
+        try:
+            if pos == 0:
+                self._dgst = _updater_dxf.get_alias(target)[0]
+            else:
+                self._dgst = 'sha256:' + target[0:pos-1]
+            self._it, self._size = _updater_dxf.pull_blob(
+                self._dgst, size=True, chunk_size=tuf.settings.CHUNK_SIZE)
+            self._it = self._it.__iter__()
+            self._end = object()
+            if _updater_progress:
+                _updater_progress(self._dgst, b'', self._size)
+        finally:
+            _updater_dxf._request_kwargs = {}
+
+    @property
+    def raw(self):
+        return self
+
+    def read(self, read_amount): #pylint: disable=unused-argument
         chunk = next(self._it, self._end)
         if chunk is self._end:
             return b''
@@ -58,14 +72,35 @@ class _DTufConnection(object):
 
     def close(self):
         # give dxf a chance to check the digest
-        if self._count == self._size:
+        if (self._it is not None) and (self._count == self._size):
             next(self._it, self._end)
 
+    def __exit__(self, *args):
+        self.close()
+
     def __str__(self):
         return "dtuf connection to %s" % self._url
 
-def _open_connection(url):
-    return _DTufConnection(url)
+class _TufSession(object): #pylint: disable=too-few-public-methods
+    def get(self, url, **kwargs): #pylint: disable=no-self-use
+        return _TufResponse(url, kwargs)
+
+class _TufSessions(object): #pylint: disable=too-few-public-methods
+    def get(self, session_index): #pylint: disable=unused-argument,no-self-use
+        return _TufSession()
+
+_tuf_sessions = _TufSessions()
+
+class _UpdaterDXF(DXF):
+    def __init__(self, *args, **kwargs):
+        super(_UpdaterDXF, self).__init__(*args, **kwargs)
+        self._request_kwargs = {}
+
+    def _base_request(self, method, path, **kwargs): #pylint: disable=redefined-outer-name
+        request_kwargs = {}
+        request_kwargs.update(self._request_kwargs)
+        request_kwargs.update(kwargs)
+        return super(_UpdaterDXF, self)._base_request(method, path, **request_kwargs)
 
 def _is_metadata_file(target):
     return target.endswith('root.json') or \
@@ -112,7 +147,8 @@ def _copy_repo_locked(f, self, *args, **kwargs):
         import tuf.settings
         import tuf.download
         # pylint: disable=protected-access
-        tuf.download._open_connection = _open_connection
+        if hasattr(tuf.download, '_sessions'):
+            tuf.download._sessions = _tuf_sessions
         global _updater_dxf
         _updater_dxf = self._dxf
         tuf.settings.repositories_directory = self._copy_dir
@@ -238,8 +274,10 @@ class _DTufCommon(DTufBase):
     # pylint: disable=too-many-arguments,super-init-not-called
     def __init__(self, host, repo, repos_root=None,
                  auth=None, insecure=False, auth_host=None):
-        self._dxf = DXF(host, repo, self._wrap_auth(auth), insecure, auth_host)
+        self._dxf = _UpdaterDXF(host, repo, self._wrap_auth(auth), insecure, auth_host)
         self._repo_root = path.join(repos_root if repos_root else path.join(getcwd(), 'dtuf_repos'), repo)
+        import tuf.settings
+        tuf.settings.FILE_HASH_ALGORITHMS = ['sha256']
 
 # pylint: disable=too-many-instance-attributes
 class DTufMaster(_DTufCommon):
@@ -322,10 +360,13 @@ class DTufMaster(_DTufCommon):
         :param password: Password to use for encrypting the private key. You'll be prompted for one if you don't supply it.
         :type password: str
         """
-        from tuf.repository_tool import generate_and_write_rsa_keypair
+        from tuf.repository_tool import generate_and_write_rsa_keypair, \
+                                        generate_and_write_rsa_keypair_with_prompt
         if password is None:
             print('generating root key...')
-        generate_and_write_rsa_keypair(self._root_key_file, password=password)
+            generate_and_write_rsa_keypair_with_prompt(filepath=self._root_key_file)
+        else:
+            generate_and_write_rsa_keypair(password, filepath=self._root_key_file)
 
     @_master_repo_locked
     def create_metadata_keys(self,
@@ -350,19 +391,26 @@ class DTufMaster(_DTufCommon):
         :param timestamp_key_password: Password to use for encrypting the TUF timestamp private key. You'll be prompted for one if you don't supply it.
         :type password: str
         """
-        from tuf.repository_tool import generate_and_write_rsa_keypair
+        from tuf.repository_tool import generate_and_write_rsa_keypair, \
+                                        generate_and_write_rsa_keypair_with_prompt
+
         if targets_key_password is None:
             print('generating targets key...')
-        generate_and_write_rsa_keypair(self._targets_key_file,
-                                       password=targets_key_password)
+            generate_and_write_rsa_keypair_with_prompt(filepath=self._targets_key_file)
+        else:
+            generate_and_write_rsa_keypair(targets_key_password, filepath=self._targets_key_file)
+
         if snapshot_key_password is None:
             print('generating snapshot key...')
-        generate_and_write_rsa_keypair(self._snapshot_key_file,
-                                       password=snapshot_key_password)
+            generate_and_write_rsa_keypair_with_prompt(filepath=self._snapshot_key_file)
+        else:
+            generate_and_write_rsa_keypair(snapshot_key_password, filepath=self._snapshot_key_file)
+
         if timestamp_key_password is None:
             print('generating timestamp key...')
-        generate_and_write_rsa_keypair(self._timestamp_key_file,
-                                       password=timestamp_key_password)
+            generate_and_write_rsa_keypair_with_prompt(filepath=self._timestamp_key_file)
+        else:
+            generate_and_write_rsa_keypair(timestamp_key_password, filepath=self._timestamp_key_file)
 
     # pylint: disable=too-many-locals
     def _add_metadata(self,
@@ -371,8 +419,9 @@ class DTufMaster(_DTufCommon):
                       targets_key_password=None,
                       snapshot_key_password=None,
                       timestamp_key_password=None):
-        from tuf.repository_tool import import_rsa_publickey_from_file, \
-                                        import_rsa_privatekey_from_file
+        from tuf.repository_tool import import_rsa_publickey_from_file
+        from tuf.repository_lib import import_rsa_privatekey_from_file
+
         # Add root key to repository
         public_root_key = import_rsa_publickey_from_file(
             self._root_key_file + '.pub')
@@ -608,7 +657,7 @@ class DTufMaster(_DTufCommon):
         # Update targets
         repository.targets.clear_targets()
         repository.targets.add_targets([
-            _strip_consistent_target_digest(f)
+            path.relpath(_strip_consistent_target_digest(f), self._master_targets_dir)
             for f in Repository.get_filepaths_in_directory(self._master_targets_dir)])
 
         # Update expirations
@@ -645,29 +694,13 @@ class DTufMaster(_DTufCommon):
         repository.write('snapshot', consistent_snapshot=True)
         repository.write('timestamp', consistent_snapshot=True)
 
-        # Upload root.json and timestamp.json without version prefix
-        for f in ['root.json', 'timestamp.json']:
-            dgst = self._dxf.push_blob(path.join(self._master_staged_dir, f),
-                                       progress)
-            self._dxf.set_alias(f, dgst)
-
-        # Upload consistent snapshot versions of current metadata files...
-        # first load timestamp.json
-        with open(path.join(self._master_staged_dir, 'timestamp.json'), 'rb') as f:
-            timestamp = json.loads(f.read().decode('utf-8'))
-        # get timestamp prefix
-        files = ['{}.timestamp.json'.format(timestamp['signed']['version'])]
-        # get snapshot prefix
-        snapshot_cs = '{}.snapshot.json'.format(timestamp['signed']['meta']['snapshot.json']['version'])
-        files.append(snapshot_cs)
-        # load prefixed snapshot.json
-        with open(path.join(self._master_staged_dir, snapshot_cs), 'rb') as f:
-            snapshot = json.loads(f.read().decode('utf-8'))
-        # get targets and root prefixes
-        files.append('{}.targets.json'.format(snapshot['signed']['meta']['targets.json']['version']))
-        files.append('{}.root.json'.format(snapshot['signed']['meta']['root.json']['version']))
-        # Upload metadata
-        for f in files:
+        # Upload metadata files
+        for f in ['root.json',
+                  'timestamp.json',
+                  '{}.root.json'.format(repository.root.version),
+                  '{}.timestamp.json'.format(repository.timestamp.version),
+                  '{}.snapshot.json'.format(repository.snapshot.version),
+                  '{}.targets.json'.format(repository.targets.version)]:
             dgst = self._dxf.push_blob(path.join(self._master_staged_dir, f),
                                        progress)
             self._dxf.set_alias(f, dgst)
@@ -790,10 +823,11 @@ class DTufCopy(_DTufCommon):
         # the public key.
         if root_public_key:
             dgst = self._dxf.get_alias('root.json')[0]
-            temp_file = securesystemslib.util.TempFile()
+            temp_file = tempfile.TemporaryFile() # pylint: disable=consider-using-with
             try:
                 it, size = self._dxf.pull_blob(dgst, size=True)
                 _write_with_progress(it, dgst, size, temp_file, progress)
+                temp_file.seek(0)
                 metadata = temp_file.read()
                 metadata_signable = json.loads(metadata.decode('utf-8'))
                 tuf.formats.check_signable_object_format(metadata_signable)
@@ -818,21 +852,25 @@ class DTufCopy(_DTufCommon):
                 })
                 if not tuf.sig.verify(metadata_signable, 'root'):
                     raise BadSignatureError('root')
-                temp_file.move(path.join(self._copy_repo_dir,
-                                         'metadata',
-                                         'current',
-                                         'root.json'))
+                securesystemslib.util.persist_temp_file(
+                    temp_file,
+                    path.join(self._copy_repo_dir,
+                              'metadata',
+                              'current',
+                              'root.json'))
             except:
-                temp_file.close_temp_file()
+                temp_file.close()
                 raise
         updater = tuf.client.updater.Updater('repository',
                                              self._repository_mirrors)
+        if hasattr(updater, 'fetcher'):
+            updater.fetcher._sessions = _tuf_sessions # pylint: disable=protected-access,no-member
         updater.refresh(False)
-        targets = updater.all_targets()
+        targets = updater._targets_of_role('targets', skip_refresh=True) # pylint: disable=protected-access
         updated_targets = updater.updated_targets(
             targets, self._copy_targets_dir)
         if path.isdir(self._copy_targets_dir):
-            targets = dict([(t['filepath'], True) for t in targets])
+            targets = {t['filepath']: True for t in targets}
             for t in listdir(self._copy_targets_dir):
                 if t not in targets:
                     remove(path.join(self._copy_targets_dir, t))
@@ -843,6 +881,8 @@ class DTufCopy(_DTufCommon):
         import tuf.client.updater
         updater = tuf.client.updater.Updater('repository',
                                              self._repository_mirrors)
+        if hasattr(updater, 'fetcher'):
+            updater.fetcher._sessions = _tuf_sessions # pylint: disable=protected-access,no-member
         tgt = updater.get_one_valid_targetinfo(target)
         updater.download_target(tgt, self._copy_targets_dir)
         with open(path.join(self._copy_targets_dir, target), 'rb') as f:
@@ -921,7 +961,9 @@ class DTufCopy(_DTufCommon):
         import tuf.client.updater
         updater = tuf.client.updater.Updater('repository',
                                              self._repository_mirrors)
-        return [t['filepath'] for t in updater.all_targets()]
+        if hasattr(updater, 'fetcher'):
+            updater.fetcher._sessions = _tuf_sessions # pylint: disable=protected-access,no-member
+        return [t['filepath'] for t in updater._targets_of_role('targets', skip_refresh=True)] # pylint: disable=protected-access
 
     @_copy_repo_locked
     def get_expirations(self):
@@ -934,6 +976,8 @@ class DTufCopy(_DTufCommon):
         import tuf.client.updater
         updater = tuf.client.updater.Updater('repository',
                                              self._repository_mirrors)
+        if hasattr(updater, 'fetcher'):
+            updater.fetcher._sessions = _tuf_sessions # pylint: disable=protected-access,no-member
         metadata = updater.metadata['current']
         return {
             'root': iso8601.parse_date(metadata['root']['expires']),
