@@ -166,9 +166,12 @@ def test_pull_metadata(dtuf_main, monkeypatch, capsys):
             # have a higher version number than the newly-created and pushed
             # master metadata. That will generate a ReplayedMetadata error.
             assert isinstance(ex2, tuf.exceptions.ReplayedMetadataError)
-            assert ex2.metadata_role == 'root'
-            assert ex2.previous_version == 1
-            assert ex2.current_version == 7
+            assert ex2.metadata_role == 'timestamp'
+            if hasattr(ex2, 'previous_version'):
+                assert ex2.previous_version == 4
+            else:
+                assert ex2.downloaded_version == 4
+            assert ex2.current_version == 15
         dir_name = path.join(dtuf_main['TEST_REPO_DIR'], pytest.repo, 'copy', 'repository', 'metadata', 'current')
         assert dir_name.startswith('/tmp/') # check what we're about to remove!
         shutil.rmtree(dir_name)
@@ -209,14 +212,12 @@ def test_see_pull_metadata_progress(dtuf_main):
     assert dtuf.main.doit(['pull-metadata', pytest.repo], environ) == 0
 
 # pylint: disable=too-many-arguments
-def _pull_target(dtuf_main, target, expected_dgsts, expected_sizes, get_info, capfd):
+def _pull_target(dtuf_main, target, expected_dgsts, expected_sizes, get_info, capfdbinary):
     environ = {'DTUF_BLOB_INFO': '1'}
     environ.update(dtuf_main)
     assert dtuf.main.doit(['pull-target', pytest.repo, target], environ if get_info else dtuf_main) == 0
     # pylint: disable=protected-access
-    encoding = capfd._capture.out.tmpfile.encoding
-    capfd._capture.out.tmpfile.encoding = None
-    out, err = capfd.readouterr()
+    out, err = capfdbinary.readouterr()
     if get_info:
         outs = BytesIO(out)
         for i, size in enumerate(expected_sizes):
@@ -233,17 +234,16 @@ def _pull_target(dtuf_main, target, expected_dgsts, expected_sizes, get_info, ca
             pos += size
             assert 'sha256:' + sha256.hexdigest() == expected_dgsts[i]
         assert pos == len(out)
-    assert err == ""
-    capfd._capture.out.tmpfile.encoding = encoding
+    assert err == b""
 
-def test_pull_target(dtuf_main, capfd):
+def test_pull_target(dtuf_main, capfdbinary):
     with pytest.raises(tuf.exceptions.UnknownTargetError):
         dtuf.main.doit(['pull-target', pytest.repo, 'dummy'], dtuf_main)
-    capfd.readouterr()
+    capfdbinary.readouterr()
     for get_info in [False, True]:
-        _pull_target(dtuf_main, 'hello', [pytest.blob1_hash], [pytest.blob1_size], get_info, capfd)
-        _pull_target(dtuf_main, 'there', [pytest.blob2_hash], [pytest.blob2_size], get_info, capfd)
-        _pull_target(dtuf_main, 'foobar', [pytest.blob1_hash, pytest.blob2_hash], [pytest.blob1_size, pytest.blob2_size], get_info, capfd)
+        _pull_target(dtuf_main, 'hello', [pytest.blob1_hash], [pytest.blob1_size], get_info, capfdbinary)
+        _pull_target(dtuf_main, 'there', [pytest.blob2_hash], [pytest.blob2_size], get_info, capfdbinary)
+        _pull_target(dtuf_main, 'foobar', [pytest.blob1_hash, pytest.blob2_hash], [pytest.blob1_size, pytest.blob2_size], get_info, capfdbinary)
 
 def test_pull_target_progress(dtuf_main, capfd):
     environ = {'DTUF_PROGRESS': '1'}
@@ -263,6 +263,8 @@ def test_see_pull_target_progress(dtuf_main, monkeypatch):
         # pylint: disable=no-self-use
         def write(self, _):
             time.sleep(0.05)
+        def flush(self):
+            pass
     monkeypatch.setattr(sys, 'stdout', FakeStdout())
     assert dtuf.main.doit(['pull-target', pytest.repo, 'hello'], environ) == 0
 
@@ -321,42 +323,6 @@ def test_auth(dtuf_main, capsys):
         assert out == ""
         assert err == ""
 
-def test_lifetime(dtuf_main, capsys):
-    for role in ['TIMESTAMP', 'SNAPSHOT', 'TARGETS', 'ROOT']:
-        environ = {
-            'DTUF_ROOT_KEY_PASSWORD': pytest.root_key_password,
-            'DTUF_TARGETS_KEY_PASSWORD': pytest.targets_key_password,
-            'DTUF_SNAPSHOT_KEY_PASSWORD': pytest.snapshot_key_password,
-            'DTUF_TIMESTAMP_KEY_PASSWORD': pytest.timestamp_key_password,
-            'DTUF_' + role + '_LIFETIME': '1s'
-        }
-        environ.update(dtuf_main)
-        assert dtuf.main.doit(['reset-keys', pytest.repo], environ) == 0
-        test_push_metadata(environ)
-        time.sleep(2)
-        with pytest.raises(tuf.exceptions.NoWorkingMirrorError) as ex:
-            dtuf.main.doit(['pull-metadata', pytest.repo], dtuf_main)
-        for ex2 in ex.value.mirror_errors.values():
-            assert isinstance(ex2, tuf.exceptions.ExpiredMetadataError)
-            assert str(ex2).startswith("Metadata u'" + role.lower() + "' expired") or \
-                   str(ex2).startswith("Metadata '" + role.lower() + "' expired")
-        capsys.readouterr()
-        dtuf.main.doit(['get-master-expirations', pytest.repo], dtuf_main)
-        out, err = capsys.readouterr()
-        assert err == ""
-        e = {}
-        for l in out.split(os.linesep):
-            if l:
-                f = l.split(': ')
-                e[f[0]] = iso8601.parse_date(f[1])
-        assert len(e) == 4
-        now = datetime.now(utc)
-        for r in ['timestamp', 'snapshot', 'targets', 'root']:
-            if r.upper() == role:
-                assert e[r] < now
-            else:
-                assert e[r] > now
-
 def test_del_target(dtuf_main, capsys):
     with pytest.raises(requests.exceptions.HTTPError) as ex:
         dtuf.main.doit(['del-target', pytest.repo, 'hello'], dtuf_main)
@@ -374,7 +340,7 @@ def test_del_target(dtuf_main, capsys):
 
 def test_reset_keys(dtuf_main, capsys):
     # create new non-root keys
-    test_create_root_key(dtuf_main)
+    test_create_metadata_keys(dtuf_main)
     # reset repository keys
     environ = {
         'DTUF_ROOT_KEY_PASSWORD': pytest.root_key_password,
@@ -387,15 +353,12 @@ def test_reset_keys(dtuf_main, capsys):
     # push metadata
     test_push_metadata(dtuf_main)
     # pull metadata
-    # this will fail even though we didn't change the root key because root.json
-    # has been updated (it stores the other public keys); unless we pass in
-    # the root public key, we don't update root.json
-    with pytest.raises(tuf.exceptions.NoWorkingMirrorError) as ex:
-        dtuf.main.doit(['pull-metadata', pytest.repo], dtuf_main)
-    for ex2 in ex.value.mirror_errors.values():
-        assert isinstance(ex2, securesystemslib.exceptions.BadSignatureError)
+    # this succeeds even though root.json has been updated (it stores the other
+    # public keys) because tuf downloads the new root.json and verifies it using
+    # the root public key from the existing root.json
+    assert dtuf.main.doit(['pull-metadata', pytest.repo], dtuf_main) == 0
     capsys.readouterr()
-    # pull metadata again with public root key
+    # pull metadata again with public root key to force verification against it
     assert _pull_metadata_with_master_public_root_key(dtuf_main) == 0
     out, err = capsys.readouterr()
     assert out == "hello2" + os.linesep
@@ -415,7 +378,8 @@ def test_reset_keys(dtuf_main, capsys):
     test_push_metadata(dtuf_main)
     # pull metadata
     # this will fail because root.json has been updated; unless we pass in the
-    # root public key, we don't update root.json
+    # root public key, the root public key from the existing root.json will
+    # be used to verify the new root.json (which fails)
     with pytest.raises(tuf.exceptions.NoWorkingMirrorError) as ex:
         dtuf.main.doit(['pull-metadata', pytest.repo], dtuf_main)
     for ex2 in ex.value.mirror_errors.values():
@@ -474,6 +438,42 @@ def test_auth_host(dtuf_main):
         with pytest.raises(requests.exceptions.ConnectionError):
             dtuf.main.doit(['list-repos'], environ)
 
+def test_lifetime(dtuf_main, capsys):
+    for role in ['TIMESTAMP', 'SNAPSHOT', 'TARGETS', 'ROOT']:
+        environ = {
+            'DTUF_ROOT_KEY_PASSWORD': pytest.root_key_password,
+            'DTUF_TARGETS_KEY_PASSWORD': pytest.targets_key_password,
+            'DTUF_SNAPSHOT_KEY_PASSWORD': pytest.snapshot_key_password,
+            'DTUF_TIMESTAMP_KEY_PASSWORD': pytest.timestamp_key_password,
+            'DTUF_' + role + '_LIFETIME': '1s'
+        }
+        environ.update(dtuf_main)
+        assert dtuf.main.doit(['reset-keys', pytest.repo], environ) == 0
+        test_push_metadata(environ)
+        time.sleep(2)
+        with pytest.raises(tuf.exceptions.NoWorkingMirrorError) as ex:
+            dtuf.main.doit(['pull-metadata', pytest.repo], dtuf_main)
+        for ex2 in ex.value.mirror_errors.values():
+            assert isinstance(ex2, tuf.exceptions.ExpiredMetadataError)
+            assert str(ex2).startswith("Metadata u'" + role.lower() + "' expired") or \
+                   str(ex2).startswith("Metadata '" + role.lower() + "' expired")
+        capsys.readouterr()
+        dtuf.main.doit(['get-master-expirations', pytest.repo], dtuf_main)
+        out, err = capsys.readouterr()
+        assert err == ""
+        e = {}
+        for l in out.split(os.linesep):
+            if l:
+                f = l.split(': ')
+                e[f[0]] = iso8601.parse_date(f[1])
+        assert len(e) == 4
+        now = datetime.now(utc)
+        for r in ['timestamp', 'snapshot', 'targets', 'root']:
+            if r.upper() == role:
+                assert e[r] < now
+            else:
+                assert e[r] > now
+
 # pylint: disable=unused-argument
 #@pytest.mark.onlytest
 def test_log(dtuf_main):
