@@ -29,8 +29,12 @@ def test_not_found(dtuf_objs):
             with pytest.raises(tuf.exceptions.NoWorkingMirrorError) as ex:
                 dtuf_objs.copy.blob_sizes(target)
             for ex2 in ex.value.mirror_errors.values():
-                assert isinstance(ex2, requests.exceptions.HTTPError)
-                assert ex2.response.status_code == requests.codes.not_found
+                if hasattr(tuf.exceptions, 'FetcherHTTPError'):
+                    assert isinstance(ex2, tuf.exceptions.FetcherHTTPError)
+                    assert ex2.status_code == requests.codes.not_found
+                else:
+                    assert isinstance(ex2, requests.exceptions.HTTPError)
+                    assert ex2.response.status_code == requests.codes.not_found
         else:
             with pytest.raises(tuf.exceptions.RepositoryError):
                 dtuf_objs.copy.blob_sizes(target)
@@ -97,9 +101,12 @@ def test_pull_metadata(dtuf_objs):
             # have a higher version number than the newly-created and pushed
             # master metadata. That will generate a ReplayedMetadata error.
             assert isinstance(ex2, tuf.exceptions.ReplayedMetadataError)
-            assert ex2.metadata_role == 'root'
-            assert ex2.previous_version == 1
-            assert ex2.current_version == 3
+            assert ex2.metadata_role == 'timestamp'
+            if hasattr(ex2, 'previous_version'):
+                assert ex2.previous_version == 2
+            else:
+                assert ex2.downloaded_version == 2
+            assert ex2.current_version == 8
         dir_name = path.join(dtuf_objs.repo_dir, pytest.repo, 'copy', 'repository', 'metadata', 'current')
         assert dir_name.startswith('/tmp/') # check what we're about to remove!
         shutil.rmtree(dir_name)
@@ -154,6 +161,27 @@ def _dummy_pull_target(dtuf_objs, target, n):
     finally:
         hashlib.sha256 = orig_sha256
 
+def test_tuf_request_kwargs(dtuf_objs):
+    session = dtuf_objs.copy._dxf._sessions[0] # actually requests module pylint: disable=protected-access
+    orig_get = session.get
+    calls = [0]
+    def get(url, **kwargs):
+        calls[0] += 1
+        # dtuf defaults
+        assert kwargs['verify']
+        assert kwargs['allow_redirects']
+        # passed from tuf
+        if calls[0] == 1:
+            assert kwargs['timeout'] == 4
+            assert kwargs['stream']
+        return orig_get(url, **kwargs)
+    session.get = get
+    try:
+        _pull_target(dtuf_objs, 'hello', [pytest.blob1_hash], None)
+    finally:
+        session.get = orig_get
+    assert calls[0] == 2 # once to download target (manifest) and once to pull blob
+
 def test_pull_target(dtuf_objs):
     with pytest.raises(tuf.exceptions.UnknownTargetError):
         for _ in dtuf_objs.copy.pull_target('dummy'):
